# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `FS.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import Ice_BuiltinSequences_ice
import omero_ServerErrors_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')
__name__ = 'omero.grid'

# Start of module omero.grid.monitors
_M_omero.grid.monitors = Ice.openModule('omero.grid.monitors')
__name__ = 'omero.grid.monitors'

if 'EventType' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.EventType = Ice.createTempClass()
    class EventType(Ice.EnumBase):
        """
        Enumeration for Monitor event types returned.
        Create, event is file or directory creation.
        Modify, event is file or directory modification.
        Delete, event is file or directory deletion.
        System, used to flag a system notification, info in fileId.
        """

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    EventType.Create = EventType("Create", 0)
    EventType.Modify = EventType("Modify", 1)
    EventType.Delete = EventType("Delete", 2)
    EventType.System = EventType("System", 3)
    EventType._enumerators = { 0:EventType.Create, 1:EventType.Modify, 2:EventType.Delete, 3:EventType.System }

    _M_omero.grid.monitors._t_EventType = IcePy.defineEnum('::omero::grid::monitors::EventType', EventType, (), EventType._enumerators)

    _M_omero.grid.monitors.EventType = EventType
    del EventType

if 'EventInfo' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.EventInfo = Ice.createTempClass()
    class EventInfo(object):
        """
        The id and type of an event. The file's basename is included for convenience,
        other stats are not included since they may be unavailable for some event types.
        """
        def __init__(self, fileId='', type=_M_omero.grid.monitors.EventType.Create):
            self.fileId = fileId
            self.type = type

        def __hash__(self):
            _h = 0
            _h = 5 * _h + Ice.getHash(self.fileId)
            _h = 5 * _h + Ice.getHash(self.type)
            return _h % 0x7fffffff

        def __compare(self, other):
            if other is None:
                return 1
            elif not isinstance(other, _M_omero.grid.monitors.EventInfo):
                return NotImplemented
            else:
                if self.fileId is None or other.fileId is None:
                    if self.fileId != other.fileId:
                        return (-1 if self.fileId is None else 1)
                else:
                    if self.fileId < other.fileId:
                        return -1
                    elif self.fileId > other.fileId:
                        return 1
                if self.type is None or other.type is None:
                    if self.type != other.type:
                        return (-1 if self.type is None else 1)
                else:
                    if self.type < other.type:
                        return -1
                    elif self.type > other.type:
                        return 1
                return 0

        def __lt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r < 0

        def __le__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r <= 0

        def __gt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r > 0

        def __ge__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r >= 0

        def __eq__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r == 0

        def __ne__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r != 0

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid.monitors._t_EventInfo)

        __repr__ = __str__

    _M_omero.grid.monitors._t_EventInfo = IcePy.defineStruct('::omero::grid::monitors::EventInfo', EventInfo, (), (
        ('fileId', (), IcePy._t_string),
        ('type', (), _M_omero.grid.monitors._t_EventType)
    ))

    _M_omero.grid.monitors.EventInfo = EventInfo
    del EventInfo

if '_t_EventList' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors._t_EventList = IcePy.defineSequence('::omero::grid::monitors::EventList', (), _M_omero.grid.monitors._t_EventInfo)

if 'MonitorClient' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.MonitorClient = Ice.createTempClass()
    class MonitorClient(Ice.Object):
        """
        This interface must be implemented by a client that
        wishes to subscribe to an OMERO.fs server.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.monitors.MonitorClient:
                raise RuntimeError('omero.grid.monitors.MonitorClient is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::monitors::MonitorClient')

        def ice_id(self, current=None):
            return '::omero::grid::monitors::MonitorClient'

        def ice_staticId():
            return '::omero::grid::monitors::MonitorClient'
        ice_staticId = staticmethod(ice_staticId)

        def fsEventHappened(self, id, el, current=None):
            """
            Callback, called by the monitor upon the proxy of the OMERO.fs client.
            Arguments:
            id -- monitor Id from which the event was reported (string).
            el -- list of events (EventList).
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid.monitors._t_MonitorClient)

        __repr__ = __str__

    _M_omero.grid.monitors.MonitorClientPrx = Ice.createTempClass()
    class MonitorClientPrx(Ice.ObjectPrx):

        """
        Callback, called by the monitor upon the proxy of the OMERO.fs client.
        Arguments:
        id -- monitor Id from which the event was reported (string).
        el -- list of events (EventList).
        _ctx -- The request context for the invocation.
        """
        def fsEventHappened(self, id, el, _ctx=None):
            return _M_omero.grid.monitors.MonitorClient._op_fsEventHappened.invoke(self, ((id, el), _ctx))

        """
        Callback, called by the monitor upon the proxy of the OMERO.fs client.
        Arguments:
        id -- monitor Id from which the event was reported (string).
        el -- list of events (EventList).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_fsEventHappened(self, id, el, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorClient._op_fsEventHappened.begin(self, ((id, el), _response, _ex, _sent, _ctx))

        """
        Callback, called by the monitor upon the proxy of the OMERO.fs client.
        Arguments:
        id -- monitor Id from which the event was reported (string).
        el -- list of events (EventList).
        """
        def end_fsEventHappened(self, _r):
            return _M_omero.grid.monitors.MonitorClient._op_fsEventHappened.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorClientPrx.ice_checkedCast(proxy, '::omero::grid::monitors::MonitorClient', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.monitors.MonitorClientPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::monitors::MonitorClient'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid.monitors._t_MonitorClientPrx = IcePy.defineProxy('::omero::grid::monitors::MonitorClient', MonitorClientPrx)

    _M_omero.grid.monitors._t_MonitorClient = IcePy.defineClass('::omero::grid::monitors::MonitorClient', MonitorClient, -1, (), True, False, None, (), ())
    MonitorClient._ice_type = _M_omero.grid.monitors._t_MonitorClient

    MonitorClient._op_fsEventHappened = IcePy.Operation('fsEventHappened', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero.grid.monitors._t_EventList, False, 0)), (), None, (_M_omero._t_ServerError,))

    _M_omero.grid.monitors.MonitorClient = MonitorClient
    del MonitorClient

    _M_omero.grid.monitors.MonitorClientPrx = MonitorClientPrx
    del MonitorClientPrx

if 'MonitorType' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.MonitorType = Ice.createTempClass()
    class MonitorType(Ice.EnumBase):

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    MonitorType.Persistent = MonitorType("Persistent", 0)
    MonitorType.OneShot = MonitorType("OneShot", 1)
    MonitorType.Inactivity = MonitorType("Inactivity", 2)
    MonitorType._enumerators = { 0:MonitorType.Persistent, 1:MonitorType.OneShot, 2:MonitorType.Inactivity }

    _M_omero.grid.monitors._t_MonitorType = IcePy.defineEnum('::omero::grid::monitors::MonitorType', MonitorType, (), MonitorType._enumerators)

    _M_omero.grid.monitors.MonitorType = MonitorType
    del MonitorType

if 'FileType' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.FileType = Ice.createTempClass()
    class FileType(Ice.EnumBase):

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    FileType.File = FileType("File", 0)
    FileType.Dir = FileType("Dir", 1)
    FileType.Link = FileType("Link", 2)
    FileType.Mount = FileType("Mount", 3)
    FileType.Unknown = FileType("Unknown", 4)
    FileType._enumerators = { 0:FileType.File, 1:FileType.Dir, 2:FileType.Link, 3:FileType.Mount, 4:FileType.Unknown }

    _M_omero.grid.monitors._t_FileType = IcePy.defineEnum('::omero::grid::monitors::FileType', FileType, (), FileType._enumerators)

    _M_omero.grid.monitors.FileType = FileType
    del FileType

if 'PathMode' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.PathMode = Ice.createTempClass()
    class PathMode(Ice.EnumBase):
        """
        Enumeration for Monitor path modes.
        Flat, monitor the specified directory but not its subdirectories.
        Recursive, monitor the specified directory and its subdirectories.
        Follow,  monitor as Recursive but with new directories being added
        to the monitor if they are created.
        Not all path modes may be implemented for a given operating system.
        """

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    PathMode.Flat = PathMode("Flat", 0)
    PathMode.Recurse = PathMode("Recurse", 1)
    PathMode.Follow = PathMode("Follow", 2)
    PathMode._enumerators = { 0:PathMode.Flat, 1:PathMode.Recurse, 2:PathMode.Follow }

    _M_omero.grid.monitors._t_PathMode = IcePy.defineEnum('::omero::grid::monitors::PathMode', PathMode, (), PathMode._enumerators)

    _M_omero.grid.monitors.PathMode = PathMode
    del PathMode

if 'WatchEventType' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.WatchEventType = Ice.createTempClass()
    class WatchEventType(Ice.EnumBase):
        """
        Enumeration for event types to watch.
        Create, notify on file creation only.
        Modify, notify on file modification only.
        Delete, notify on file deletion only.
        All, notify on all vents in the enumeration that apply to a given OS.
        Not all event types may be implemented for a given operating system.
        """

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    WatchEventType.Creation = WatchEventType("Creation", 0)
    WatchEventType.Modification = WatchEventType("Modification", 1)
    WatchEventType.Deletion = WatchEventType("Deletion", 2)
    WatchEventType.All = WatchEventType("All", 3)
    WatchEventType._enumerators = { 0:WatchEventType.Creation, 1:WatchEventType.Modification, 2:WatchEventType.Deletion, 3:WatchEventType.All }

    _M_omero.grid.monitors._t_WatchEventType = IcePy.defineEnum('::omero::grid::monitors::WatchEventType', WatchEventType, (), WatchEventType._enumerators)

    _M_omero.grid.monitors.WatchEventType = WatchEventType
    del WatchEventType

if 'MonitorState' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.MonitorState = Ice.createTempClass()
    class MonitorState(Ice.EnumBase):
        """
        Enumeration for Monitor state.
        Stopped, a monitor exists but is not actively monitoring.
        Started, a monitor exists and is actively monitoring.
        """

        def __init__(self, _n, _v):
            Ice.EnumBase.__init__(self, _n, _v)

        def valueOf(self, _n):
            if _n in self._enumerators:
                return self._enumerators[_n]
            return None
        valueOf = classmethod(valueOf)

    MonitorState.Stopped = MonitorState("Stopped", 0)
    MonitorState.Started = MonitorState("Started", 1)
    MonitorState._enumerators = { 0:MonitorState.Stopped, 1:MonitorState.Started }

    _M_omero.grid.monitors._t_MonitorState = IcePy.defineEnum('::omero::grid::monitors::MonitorState', MonitorState, (), MonitorState._enumerators)

    _M_omero.grid.monitors.MonitorState = MonitorState
    del MonitorState

if 'FileStats' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.FileStats = Ice.createTempClass()
    class FileStats(object):
        """
        File stats.
        What stats are likely to be needed? Could this struct be trimmed down
        or does it need any further attributes?
        """
        def __init__(self, baseName='', owner='', size=0, mTime=0.0, cTime=0.0, aTime=0.0, type=_M_omero.grid.monitors.FileType.File):
            self.baseName = baseName
            self.owner = owner
            self.size = size
            self.mTime = mTime
            self.cTime = cTime
            self.aTime = aTime
            self.type = type

        def __eq__(self, other):
            if other is None:
                return False
            elif not isinstance(other, _M_omero.grid.monitors.FileStats):
                return NotImplemented
            else:
                if self.baseName != other.baseName:
                    return False
                if self.owner != other.owner:
                    return False
                if self.size != other.size:
                    return False
                if self.mTime != other.mTime:
                    return False
                if self.cTime != other.cTime:
                    return False
                if self.aTime != other.aTime:
                    return False
                if self.type != other.type:
                    return False
                return True

        def __ne__(self, other):
            return not self.__eq__(other)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid.monitors._t_FileStats)

        __repr__ = __str__

    _M_omero.grid.monitors._t_FileStats = IcePy.defineStruct('::omero::grid::monitors::FileStats', FileStats, (), (
        ('baseName', (), IcePy._t_string),
        ('owner', (), IcePy._t_string),
        ('size', (), IcePy._t_long),
        ('mTime', (), IcePy._t_float),
        ('cTime', (), IcePy._t_float),
        ('aTime', (), IcePy._t_float),
        ('type', (), _M_omero.grid.monitors._t_FileType)
    ))

    _M_omero.grid.monitors.FileStats = FileStats
    del FileStats

if '_t_WatchEventList' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors._t_WatchEventList = IcePy.defineSequence('::omero::grid::monitors::WatchEventList', (), _M_omero.grid.monitors._t_WatchEventType)

if '_t_FileStatsList' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors._t_FileStatsList = IcePy.defineSequence('::omero::grid::monitors::FileStatsList', (), _M_omero.grid.monitors._t_FileStats)

if 'FileServer' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.FileServer = Ice.createTempClass()
    class FileServer(Ice.Object):
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.monitors.FileServer:
                raise RuntimeError('omero.grid.monitors.FileServer is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::monitors::FileServer')

        def ice_id(self, current=None):
            return '::omero::grid::monitors::FileServer'

        def ice_staticId():
            return '::omero::grid::monitors::FileServer'
        ice_staticId = staticmethod(ice_staticId)

        def getDirectory(self, absPath, filter, current=None):
            """
            Get an absolute directory from an OMERO.fs server.
            The returned list will contain just the file names for each directory entry.
            An exception will be raised if the path does not exist or is inaccessible to the
            OMERO.fs server. An exception will be raised if directory list cannot be
            returned for any other reason.
            Arguments:
            absPath -- an absolute path on the monitor's watch path (string).
            filter -- a filter to apply to the listing, cf. ls (string).
            current -- The Current object for the invocation.
            Returns: a directory listing (Ice::StringSeq).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getBulkDirectory(self, absPath, filter, current=None):
            """
            Get an absolute directory from an OMERO.fs server.
            The returned list will contain the file stats for each directory entry.
            An exception will be raised if the path does not exist or is inaccessible to the
            OMERO.fs server. An exception will be raised if directory list cannot be
            returned for any other reason.
            Arguments:
            absPath -- an absolute path on the monitor's watch path (string).
            filter -- a filter to apply to the listing, cf. ls (string).
            current -- The Current object for the invocation.
            Returns: a directory listing (FileStatsList).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def fileExists(self, fileId, current=None):
            """
            Query the existence of a file
            An exception will be raised if the method fails to determine the existence.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: existence of file.
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getBaseName(self, fileId, current=None):
            """
            Get base name of a file, this is the name
            stripped of any path, e.g. file.ext
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: base name.
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getStats(self, fileId, current=None):
            """
            Get all FileStats of a file
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: file stats (FileStats).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getSize(self, fileId, current=None):
            """
            Get size of a file in bytes
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: byte size of file (long).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getOwner(self, fileId, current=None):
            """
            Get owner of a file
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: owner of file (string).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getCTime(self, fileId, current=None):
            """
            Get ctime of a file
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: ctime of file (float).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getMTime(self, fileId, current=None):
            """
            Get mtime of a file
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: mtime of file (float).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getATime(self, fileId, current=None):
            """
            Get atime of a file
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: atime of file (float).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def isDir(self, fileId, current=None):
            """
            Query whether file is a directory
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: true is directory (bool).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def isFile(self, fileId, current=None):
            """
            Query whether file is a file
            An exception will be raised if the file no longer exists or is inaccessible.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: true if file (bool).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getSHA1(self, fileId, current=None):
            """
            Get SHA1 of a file
            An exception will be raised if the file no longer exists or is inaccessible.
            An exception will be raised if the SHA1 cannot be generated for any reason.
            Arguments:
            fileId -- see above.
            current -- The Current object for the invocation.
            Returns: SHA1 hex hash digest of file (string).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def readBlock(self, fileId, offset, size, current=None):
            """
            readBlock should open, read size bytes from offset
            and then close the file.
            An exception will be raised if the file no longer exists or is inaccessible.
            An exception will be raised if the file read fails for any other reason.
            Arguments:
            fileId -- see above.
            offset -- byte offset into file from where read should begin (long).
            size -- number of bytes that should be read (int).
            current -- The Current object for the invocation.
            Returns: byte sequence of upto size bytes.
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid.monitors._t_FileServer)

        __repr__ = __str__

    _M_omero.grid.monitors.FileServerPrx = Ice.createTempClass()
    class FileServerPrx(Ice.ObjectPrx):

        """
        Get an absolute directory from an OMERO.fs server.
        The returned list will contain just the file names for each directory entry.
        An exception will be raised if the path does not exist or is inaccessible to the
        OMERO.fs server. An exception will be raised if directory list cannot be
        returned for any other reason.
        Arguments:
        absPath -- an absolute path on the monitor's watch path (string).
        filter -- a filter to apply to the listing, cf. ls (string).
        _ctx -- The request context for the invocation.
        Returns: a directory listing (Ice::StringSeq).
        Throws:
        omero::OmeroFSError -- 
        """
        def getDirectory(self, absPath, filter, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getDirectory.invoke(self, ((absPath, filter), _ctx))

        """
        Get an absolute directory from an OMERO.fs server.
        The returned list will contain just the file names for each directory entry.
        An exception will be raised if the path does not exist or is inaccessible to the
        OMERO.fs server. An exception will be raised if directory list cannot be
        returned for any other reason.
        Arguments:
        absPath -- an absolute path on the monitor's watch path (string).
        filter -- a filter to apply to the listing, cf. ls (string).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getDirectory(self, absPath, filter, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getDirectory.begin(self, ((absPath, filter), _response, _ex, _sent, _ctx))

        """
        Get an absolute directory from an OMERO.fs server.
        The returned list will contain just the file names for each directory entry.
        An exception will be raised if the path does not exist or is inaccessible to the
        OMERO.fs server. An exception will be raised if directory list cannot be
        returned for any other reason.
        Arguments:
        absPath -- an absolute path on the monitor's watch path (string).
        filter -- a filter to apply to the listing, cf. ls (string).
        Returns: a directory listing (Ice::StringSeq).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getDirectory(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getDirectory.end(self, _r)

        """
        Get an absolute directory from an OMERO.fs server.
        The returned list will contain the file stats for each directory entry.
        An exception will be raised if the path does not exist or is inaccessible to the
        OMERO.fs server. An exception will be raised if directory list cannot be
        returned for any other reason.
        Arguments:
        absPath -- an absolute path on the monitor's watch path (string).
        filter -- a filter to apply to the listing, cf. ls (string).
        _ctx -- The request context for the invocation.
        Returns: a directory listing (FileStatsList).
        Throws:
        omero::OmeroFSError -- 
        """
        def getBulkDirectory(self, absPath, filter, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getBulkDirectory.invoke(self, ((absPath, filter), _ctx))

        """
        Get an absolute directory from an OMERO.fs server.
        The returned list will contain the file stats for each directory entry.
        An exception will be raised if the path does not exist or is inaccessible to the
        OMERO.fs server. An exception will be raised if directory list cannot be
        returned for any other reason.
        Arguments:
        absPath -- an absolute path on the monitor's watch path (string).
        filter -- a filter to apply to the listing, cf. ls (string).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getBulkDirectory(self, absPath, filter, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getBulkDirectory.begin(self, ((absPath, filter), _response, _ex, _sent, _ctx))

        """
        Get an absolute directory from an OMERO.fs server.
        The returned list will contain the file stats for each directory entry.
        An exception will be raised if the path does not exist or is inaccessible to the
        OMERO.fs server. An exception will be raised if directory list cannot be
        returned for any other reason.
        Arguments:
        absPath -- an absolute path on the monitor's watch path (string).
        filter -- a filter to apply to the listing, cf. ls (string).
        Returns: a directory listing (FileStatsList).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getBulkDirectory(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getBulkDirectory.end(self, _r)

        """
        Query the existence of a file
        An exception will be raised if the method fails to determine the existence.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: existence of file.
        Throws:
        omero::OmeroFSError -- 
        """
        def fileExists(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_fileExists.invoke(self, ((fileId, ), _ctx))

        """
        Query the existence of a file
        An exception will be raised if the method fails to determine the existence.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_fileExists(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_fileExists.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Query the existence of a file
        An exception will be raised if the method fails to determine the existence.
        Arguments:
        fileId -- see above.
        Returns: existence of file.
        Throws:
        omero::OmeroFSError -- 
        """
        def end_fileExists(self, _r):
            return _M_omero.grid.monitors.FileServer._op_fileExists.end(self, _r)

        """
        Get base name of a file, this is the name
        stripped of any path, e.g. file.ext
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: base name.
        Throws:
        omero::OmeroFSError -- 
        """
        def getBaseName(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getBaseName.invoke(self, ((fileId, ), _ctx))

        """
        Get base name of a file, this is the name
        stripped of any path, e.g. file.ext
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getBaseName(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getBaseName.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get base name of a file, this is the name
        stripped of any path, e.g. file.ext
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: base name.
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getBaseName(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getBaseName.end(self, _r)

        """
        Get all FileStats of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: file stats (FileStats).
        Throws:
        omero::OmeroFSError -- 
        """
        def getStats(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getStats.invoke(self, ((fileId, ), _ctx))

        """
        Get all FileStats of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getStats(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getStats.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get all FileStats of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: file stats (FileStats).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getStats(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getStats.end(self, _r)

        """
        Get size of a file in bytes
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: byte size of file (long).
        Throws:
        omero::OmeroFSError -- 
        """
        def getSize(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getSize.invoke(self, ((fileId, ), _ctx))

        """
        Get size of a file in bytes
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getSize(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getSize.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get size of a file in bytes
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: byte size of file (long).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getSize(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getSize.end(self, _r)

        """
        Get owner of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: owner of file (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def getOwner(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getOwner.invoke(self, ((fileId, ), _ctx))

        """
        Get owner of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getOwner(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getOwner.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get owner of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: owner of file (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getOwner(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getOwner.end(self, _r)

        """
        Get ctime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: ctime of file (float).
        Throws:
        omero::OmeroFSError -- 
        """
        def getCTime(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getCTime.invoke(self, ((fileId, ), _ctx))

        """
        Get ctime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getCTime(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getCTime.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get ctime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: ctime of file (float).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getCTime(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getCTime.end(self, _r)

        """
        Get mtime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: mtime of file (float).
        Throws:
        omero::OmeroFSError -- 
        """
        def getMTime(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getMTime.invoke(self, ((fileId, ), _ctx))

        """
        Get mtime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMTime(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getMTime.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get mtime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: mtime of file (float).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getMTime(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getMTime.end(self, _r)

        """
        Get atime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: atime of file (float).
        Throws:
        omero::OmeroFSError -- 
        """
        def getATime(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getATime.invoke(self, ((fileId, ), _ctx))

        """
        Get atime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getATime(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getATime.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get atime of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: atime of file (float).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getATime(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getATime.end(self, _r)

        """
        Query whether file is a directory
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: true is directory (bool).
        Throws:
        omero::OmeroFSError -- 
        """
        def isDir(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_isDir.invoke(self, ((fileId, ), _ctx))

        """
        Query whether file is a directory
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isDir(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_isDir.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Query whether file is a directory
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: true is directory (bool).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_isDir(self, _r):
            return _M_omero.grid.monitors.FileServer._op_isDir.end(self, _r)

        """
        Query whether file is a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: true if file (bool).
        Throws:
        omero::OmeroFSError -- 
        """
        def isFile(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_isFile.invoke(self, ((fileId, ), _ctx))

        """
        Query whether file is a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isFile(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_isFile.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Query whether file is a file
        An exception will be raised if the file no longer exists or is inaccessible.
        Arguments:
        fileId -- see above.
        Returns: true if file (bool).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_isFile(self, _r):
            return _M_omero.grid.monitors.FileServer._op_isFile.end(self, _r)

        """
        Get SHA1 of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        An exception will be raised if the SHA1 cannot be generated for any reason.
        Arguments:
        fileId -- see above.
        _ctx -- The request context for the invocation.
        Returns: SHA1 hex hash digest of file (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def getSHA1(self, fileId, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getSHA1.invoke(self, ((fileId, ), _ctx))

        """
        Get SHA1 of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        An exception will be raised if the SHA1 cannot be generated for any reason.
        Arguments:
        fileId -- see above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getSHA1(self, fileId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_getSHA1.begin(self, ((fileId, ), _response, _ex, _sent, _ctx))

        """
        Get SHA1 of a file
        An exception will be raised if the file no longer exists or is inaccessible.
        An exception will be raised if the SHA1 cannot be generated for any reason.
        Arguments:
        fileId -- see above.
        Returns: SHA1 hex hash digest of file (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getSHA1(self, _r):
            return _M_omero.grid.monitors.FileServer._op_getSHA1.end(self, _r)

        """
        readBlock should open, read size bytes from offset
        and then close the file.
        An exception will be raised if the file no longer exists or is inaccessible.
        An exception will be raised if the file read fails for any other reason.
        Arguments:
        fileId -- see above.
        offset -- byte offset into file from where read should begin (long).
        size -- number of bytes that should be read (int).
        _ctx -- The request context for the invocation.
        Returns: byte sequence of upto size bytes.
        Throws:
        omero::OmeroFSError -- 
        """
        def readBlock(self, fileId, offset, size, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_readBlock.invoke(self, ((fileId, offset, size), _ctx))

        """
        readBlock should open, read size bytes from offset
        and then close the file.
        An exception will be raised if the file no longer exists or is inaccessible.
        An exception will be raised if the file read fails for any other reason.
        Arguments:
        fileId -- see above.
        offset -- byte offset into file from where read should begin (long).
        size -- number of bytes that should be read (int).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_readBlock(self, fileId, offset, size, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.FileServer._op_readBlock.begin(self, ((fileId, offset, size), _response, _ex, _sent, _ctx))

        """
        readBlock should open, read size bytes from offset
        and then close the file.
        An exception will be raised if the file no longer exists or is inaccessible.
        An exception will be raised if the file read fails for any other reason.
        Arguments:
        fileId -- see above.
        offset -- byte offset into file from where read should begin (long).
        size -- number of bytes that should be read (int).
        Returns: byte sequence of upto size bytes.
        Throws:
        omero::OmeroFSError -- 
        """
        def end_readBlock(self, _r):
            return _M_omero.grid.monitors.FileServer._op_readBlock.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.monitors.FileServerPrx.ice_checkedCast(proxy, '::omero::grid::monitors::FileServer', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.monitors.FileServerPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::monitors::FileServer'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid.monitors._t_FileServerPrx = IcePy.defineProxy('::omero::grid::monitors::FileServer', FileServerPrx)

    _M_omero.grid.monitors._t_FileServer = IcePy.defineClass('::omero::grid::monitors::FileServer', FileServer, -1, (), True, False, None, (), ())
    FileServer._ice_type = _M_omero.grid.monitors._t_FileServer

    FileServer._op_getDirectory = IcePy.Operation('getDirectory', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_Ice._t_StringSeq, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getBulkDirectory = IcePy.Operation('getBulkDirectory', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_omero.grid.monitors._t_FileStatsList, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_fileExists = IcePy.Operation('fileExists', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getBaseName = IcePy.Operation('getBaseName', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getStats = IcePy.Operation('getStats', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.grid.monitors._t_FileStats, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getSize = IcePy.Operation('getSize', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_long, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getOwner = IcePy.Operation('getOwner', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getCTime = IcePy.Operation('getCTime', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_float, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getMTime = IcePy.Operation('getMTime', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_float, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getATime = IcePy.Operation('getATime', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_float, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_isDir = IcePy.Operation('isDir', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_isFile = IcePy.Operation('isFile', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_getSHA1 = IcePy.Operation('getSHA1', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), IcePy._t_string, False, 0), (_M_omero._t_OmeroFSError,))
    FileServer._op_readBlock = IcePy.Operation('readBlock', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_long, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_OmeroFSError,))

    _M_omero.grid.monitors.FileServer = FileServer
    del FileServer

    _M_omero.grid.monitors.FileServerPrx = FileServerPrx
    del FileServerPrx

if 'MonitorServer' not in _M_omero.grid.monitors.__dict__:
    _M_omero.grid.monitors.MonitorServer = Ice.createTempClass()
    class MonitorServer(Ice.Object):
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.monitors.MonitorServer:
                raise RuntimeError('omero.grid.monitors.MonitorServer is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::monitors::MonitorServer')

        def ice_id(self, current=None):
            return '::omero::grid::monitors::MonitorServer'

        def ice_staticId():
            return '::omero::grid::monitors::MonitorServer'
        ice_staticId = staticmethod(ice_staticId)

        def createMonitor(self, mType, eTypes, pMode, pathString, whitelist, blacklist, timeout, blockSize, ignoreSysFiles, ignoreDirEvents, platformCheck, proxy, current=None):
            """
            Create a monitor of events.
            A exception will be raised if the event type or path mode is not supported by
            the Monitor implementation for a given OS. An exception will be raised if the
            path does not exist or is inaccessible to the monitor. An exception will be raised
            if a monitor cannot be created for any other reason.
            Arguments:
            mType -- type of monitor to create (MonitorType).
            eTypes -- a sequence of watch event type to monitor (WatchEventTypeList).
            pMode -- path mode of monitor (PathMode).
            pathString -- full path of directory of interest (string).
            whitelist -- list of files or extensions of interest (Ice::StringSeq).
            blacklist -- list of directories, files or extensions that are not of interest (Ice::StringSeq).
            timeout -- time in seconds fo monitor to time out (float).
            blockSize -- the number of events to pack into each notification (int).
            ignoreSysFiles -- ignore system files or not (bool).
            ignoreDirEvents -- ignore directory events (bool).
            platformCheck -- if true strictly check platform (bool).
            proxy -- a proxy of the client to which notifications will be sent (MonitorClient*).
            current -- The Current object for the invocation.
            Returns: monitorId, a uuid1 (string).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def startMonitor(self, id, current=None):
            """
            Start an existing monitor.
            An exception will be raised if the id does not correspond to an existing monitor.
            An exception will be raised if a monitor cannot be started for any other reason,
            in this case the monitor's state cannot be assumed.
            Arguments:
            id -- monitor id (string).
            current -- The Current object for the invocation.
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def stopMonitor(self, id, current=None):
            """
            Stop an existing monitor.
            Attempting to stop a monitor that is not running raises no exception.
            An exception will be raised if the id does not correspond to an existing monitor.
            An exception will be raised if a monitor cannot be stopped for any other reason,
            in this case the monitor's state cannot be assumed.
            Arguments:
            id -- monitor id (string).
            current -- The Current object for the invocation.
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def destroyMonitor(self, id, current=None):
            """
            Destroy an existing monitor.
            Attempting to destroy a monitor that is running will try to first stop
            the monitor and then destroy it.
            An exception will be raised if the id does not correspond to an existing monitor.
            An exception will be raised if a monitor cannot be destroyed (or stopped and destroyed)
            for any other reason, in this case the monitor's state cannot be assumed.
            Arguments:
            id -- monitor id (string).
            current -- The Current object for the invocation.
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def getMonitorState(self, id, current=None):
            """
            Get the state of an existing monitor.
            An exception will be raised if the id does not correspond to an existing monitor.
            Arguments:
            id -- monitor id (string).
            current -- The Current object for the invocation.
            Returns: the monitor state (MonitorState).
            Throws:
            omero::OmeroFSError -- 
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid.monitors._t_MonitorServer)

        __repr__ = __str__

    _M_omero.grid.monitors.MonitorServerPrx = Ice.createTempClass()
    class MonitorServerPrx(Ice.ObjectPrx):

        """
        Create a monitor of events.
        A exception will be raised if the event type or path mode is not supported by
        the Monitor implementation for a given OS. An exception will be raised if the
        path does not exist or is inaccessible to the monitor. An exception will be raised
        if a monitor cannot be created for any other reason.
        Arguments:
        mType -- type of monitor to create (MonitorType).
        eTypes -- a sequence of watch event type to monitor (WatchEventTypeList).
        pMode -- path mode of monitor (PathMode).
        pathString -- full path of directory of interest (string).
        whitelist -- list of files or extensions of interest (Ice::StringSeq).
        blacklist -- list of directories, files or extensions that are not of interest (Ice::StringSeq).
        timeout -- time in seconds fo monitor to time out (float).
        blockSize -- the number of events to pack into each notification (int).
        ignoreSysFiles -- ignore system files or not (bool).
        ignoreDirEvents -- ignore directory events (bool).
        platformCheck -- if true strictly check platform (bool).
        proxy -- a proxy of the client to which notifications will be sent (MonitorClient*).
        _ctx -- The request context for the invocation.
        Returns: monitorId, a uuid1 (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def createMonitor(self, mType, eTypes, pMode, pathString, whitelist, blacklist, timeout, blockSize, ignoreSysFiles, ignoreDirEvents, platformCheck, proxy, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_createMonitor.invoke(self, ((mType, eTypes, pMode, pathString, whitelist, blacklist, timeout, blockSize, ignoreSysFiles, ignoreDirEvents, platformCheck, proxy), _ctx))

        """
        Create a monitor of events.
        A exception will be raised if the event type or path mode is not supported by
        the Monitor implementation for a given OS. An exception will be raised if the
        path does not exist or is inaccessible to the monitor. An exception will be raised
        if a monitor cannot be created for any other reason.
        Arguments:
        mType -- type of monitor to create (MonitorType).
        eTypes -- a sequence of watch event type to monitor (WatchEventTypeList).
        pMode -- path mode of monitor (PathMode).
        pathString -- full path of directory of interest (string).
        whitelist -- list of files or extensions of interest (Ice::StringSeq).
        blacklist -- list of directories, files or extensions that are not of interest (Ice::StringSeq).
        timeout -- time in seconds fo monitor to time out (float).
        blockSize -- the number of events to pack into each notification (int).
        ignoreSysFiles -- ignore system files or not (bool).
        ignoreDirEvents -- ignore directory events (bool).
        platformCheck -- if true strictly check platform (bool).
        proxy -- a proxy of the client to which notifications will be sent (MonitorClient*).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createMonitor(self, mType, eTypes, pMode, pathString, whitelist, blacklist, timeout, blockSize, ignoreSysFiles, ignoreDirEvents, platformCheck, proxy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_createMonitor.begin(self, ((mType, eTypes, pMode, pathString, whitelist, blacklist, timeout, blockSize, ignoreSysFiles, ignoreDirEvents, platformCheck, proxy), _response, _ex, _sent, _ctx))

        """
        Create a monitor of events.
        A exception will be raised if the event type or path mode is not supported by
        the Monitor implementation for a given OS. An exception will be raised if the
        path does not exist or is inaccessible to the monitor. An exception will be raised
        if a monitor cannot be created for any other reason.
        Arguments:
        mType -- type of monitor to create (MonitorType).
        eTypes -- a sequence of watch event type to monitor (WatchEventTypeList).
        pMode -- path mode of monitor (PathMode).
        pathString -- full path of directory of interest (string).
        whitelist -- list of files or extensions of interest (Ice::StringSeq).
        blacklist -- list of directories, files or extensions that are not of interest (Ice::StringSeq).
        timeout -- time in seconds fo monitor to time out (float).
        blockSize -- the number of events to pack into each notification (int).
        ignoreSysFiles -- ignore system files or not (bool).
        ignoreDirEvents -- ignore directory events (bool).
        platformCheck -- if true strictly check platform (bool).
        proxy -- a proxy of the client to which notifications will be sent (MonitorClient*).
        Returns: monitorId, a uuid1 (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_createMonitor(self, _r):
            return _M_omero.grid.monitors.MonitorServer._op_createMonitor.end(self, _r)

        """
        Start an existing monitor.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be started for any other reason,
        in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        _ctx -- The request context for the invocation.
        Throws:
        omero::OmeroFSError -- 
        """
        def startMonitor(self, id, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_startMonitor.invoke(self, ((id, ), _ctx))

        """
        Start an existing monitor.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be started for any other reason,
        in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_startMonitor(self, id, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_startMonitor.begin(self, ((id, ), _response, _ex, _sent, _ctx))

        """
        Start an existing monitor.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be started for any other reason,
        in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_startMonitor(self, _r):
            return _M_omero.grid.monitors.MonitorServer._op_startMonitor.end(self, _r)

        """
        Stop an existing monitor.
        Attempting to stop a monitor that is not running raises no exception.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be stopped for any other reason,
        in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        _ctx -- The request context for the invocation.
        Throws:
        omero::OmeroFSError -- 
        """
        def stopMonitor(self, id, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_stopMonitor.invoke(self, ((id, ), _ctx))

        """
        Stop an existing monitor.
        Attempting to stop a monitor that is not running raises no exception.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be stopped for any other reason,
        in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_stopMonitor(self, id, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_stopMonitor.begin(self, ((id, ), _response, _ex, _sent, _ctx))

        """
        Stop an existing monitor.
        Attempting to stop a monitor that is not running raises no exception.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be stopped for any other reason,
        in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_stopMonitor(self, _r):
            return _M_omero.grid.monitors.MonitorServer._op_stopMonitor.end(self, _r)

        """
        Destroy an existing monitor.
        Attempting to destroy a monitor that is running will try to first stop
        the monitor and then destroy it.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be destroyed (or stopped and destroyed)
        for any other reason, in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        _ctx -- The request context for the invocation.
        Throws:
        omero::OmeroFSError -- 
        """
        def destroyMonitor(self, id, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_destroyMonitor.invoke(self, ((id, ), _ctx))

        """
        Destroy an existing monitor.
        Attempting to destroy a monitor that is running will try to first stop
        the monitor and then destroy it.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be destroyed (or stopped and destroyed)
        for any other reason, in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_destroyMonitor(self, id, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_destroyMonitor.begin(self, ((id, ), _response, _ex, _sent, _ctx))

        """
        Destroy an existing monitor.
        Attempting to destroy a monitor that is running will try to first stop
        the monitor and then destroy it.
        An exception will be raised if the id does not correspond to an existing monitor.
        An exception will be raised if a monitor cannot be destroyed (or stopped and destroyed)
        for any other reason, in this case the monitor's state cannot be assumed.
        Arguments:
        id -- monitor id (string).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_destroyMonitor(self, _r):
            return _M_omero.grid.monitors.MonitorServer._op_destroyMonitor.end(self, _r)

        """
        Get the state of an existing monitor.
        An exception will be raised if the id does not correspond to an existing monitor.
        Arguments:
        id -- monitor id (string).
        _ctx -- The request context for the invocation.
        Returns: the monitor state (MonitorState).
        Throws:
        omero::OmeroFSError -- 
        """
        def getMonitorState(self, id, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_getMonitorState.invoke(self, ((id, ), _ctx))

        """
        Get the state of an existing monitor.
        An exception will be raised if the id does not correspond to an existing monitor.
        Arguments:
        id -- monitor id (string).
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMonitorState(self, id, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorServer._op_getMonitorState.begin(self, ((id, ), _response, _ex, _sent, _ctx))

        """
        Get the state of an existing monitor.
        An exception will be raised if the id does not correspond to an existing monitor.
        Arguments:
        id -- monitor id (string).
        Returns: the monitor state (MonitorState).
        Throws:
        omero::OmeroFSError -- 
        """
        def end_getMonitorState(self, _r):
            return _M_omero.grid.monitors.MonitorServer._op_getMonitorState.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.monitors.MonitorServerPrx.ice_checkedCast(proxy, '::omero::grid::monitors::MonitorServer', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.monitors.MonitorServerPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::monitors::MonitorServer'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid.monitors._t_MonitorServerPrx = IcePy.defineProxy('::omero::grid::monitors::MonitorServer', MonitorServerPrx)

    _M_omero.grid.monitors._t_MonitorServer = IcePy.defineClass('::omero::grid::monitors::MonitorServer', MonitorServer, -1, (), True, False, None, (), ())
    MonitorServer._ice_type = _M_omero.grid.monitors._t_MonitorServer

    MonitorServer._op_createMonitor = IcePy.Operation('createMonitor', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.grid.monitors._t_MonitorType, False, 0), ((), _M_omero.grid.monitors._t_WatchEventList, False, 0), ((), _M_omero.grid.monitors._t_PathMode, False, 0), ((), IcePy._t_string, False, 0), ((), _M_Ice._t_StringSeq, False, 0), ((), _M_Ice._t_StringSeq, False, 0), ((), IcePy._t_float, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_bool, False, 0), ((), IcePy._t_bool, False, 0), ((), IcePy._t_bool, False, 0), ((), _M_omero.grid.monitors._t_MonitorClientPrx, False, 0)), (), ((), IcePy._t_string, False, 0), (_M_omero._t_OmeroFSError,))
    MonitorServer._op_startMonitor = IcePy.Operation('startMonitor', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), None, (_M_omero._t_OmeroFSError,))
    MonitorServer._op_stopMonitor = IcePy.Operation('stopMonitor', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), None, (_M_omero._t_OmeroFSError,))
    MonitorServer._op_destroyMonitor = IcePy.Operation('destroyMonitor', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), None, (_M_omero._t_OmeroFSError,))
    MonitorServer._op_getMonitorState = IcePy.Operation('getMonitorState', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.grid.monitors._t_MonitorState, False, 0), (_M_omero._t_OmeroFSError,))

    _M_omero.grid.monitors.MonitorServer = MonitorServer
    del MonitorServer

    _M_omero.grid.monitors.MonitorServerPrx = MonitorServerPrx
    del MonitorServerPrx

# End of module omero.grid.monitors

__name__ = 'omero.grid'

# End of module omero.grid

__name__ = 'omero'

# End of module omero



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_ThumbnailGenerationJob_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class ThumbnailGenerationJobI(_omero_model.ThumbnailGenerationJob):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "username",
          "groupname",
          "type",
          "message",
          "status",
          "submitted",
          "scheduledFor",
          "started",
          "finished",
          "originalFileLinks",
          "details",
      ])
      _field_info = _field_info_type(
          username=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          groupname=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          type=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          message=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          status=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          submitted=_field_info_data(wrapper=omero.rtypes.rtime, nullable=False),
          scheduledFor=_field_info_data(wrapper=omero.rtypes.rtime, nullable=False),
          started=_field_info_data(wrapper=omero.rtypes.rtime, nullable=True),
          finished=_field_info_data(wrapper=omero.rtypes.rtime, nullable=True),
          originalFileLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      USERNAME =  "ome.model.jobs.ThumbnailGenerationJob_username"
      GROUPNAME =  "ome.model.jobs.ThumbnailGenerationJob_groupname"
      TYPE =  "ome.model.jobs.ThumbnailGenerationJob_type"
      MESSAGE =  "ome.model.jobs.ThumbnailGenerationJob_message"
      STATUS =  "ome.model.jobs.ThumbnailGenerationJob_status"
      SUBMITTED =  "ome.model.jobs.ThumbnailGenerationJob_submitted"
      SCHEDULEDFOR =  "ome.model.jobs.ThumbnailGenerationJob_scheduledFor"
      STARTED =  "ome.model.jobs.ThumbnailGenerationJob_started"
      FINISHED =  "ome.model.jobs.ThumbnailGenerationJob_finished"
      ORIGINALFILELINKS =  "ome.model.jobs.ThumbnailGenerationJob_originalFileLinks"
      DETAILS =  "ome.model.jobs.ThumbnailGenerationJob_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          if load:
              self._originalFileLinksSeq = []
              self._originalFileLinksLoaded = True;
          else:
              self._originalFileLinksSeq = []
              self._originalFileLinksLoaded = False;

          pass

      def __init__(self, id=None, loaded=None):
          super(ThumbnailGenerationJobI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadUsername( )
          self.unloadGroupname( )
          self.unloadType( )
          self.unloadMessage( )
          self.unloadStatus( )
          self.unloadSubmitted( )
          self.unloadScheduledFor( )
          self.unloadStarted( )
          self.unloadFinished( )
          self.unloadOriginalFileLinks( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  False ;
      def isMutable(self, current = None):
          return  True ;
      def isAnnotated(self, current = None):
          return  False ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = ThumbnailGenerationJobI()
            copy._id = self._id;
            copy._version = self._version;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return ThumbnailGenerationJobI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def getVersion(self, current = None):
          self.errorIfUnloaded()
          return self._version

      def setVersion(self, version, current = None):
          self.errorIfUnloaded()
          self._version = version

      def unloadUsername(self, ):
          self._usernameLoaded = False
          self._username = None;

      def getUsername(self, current = None):
          self.errorIfUnloaded()
          return self._username

      def setUsername(self, _username, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.username.wrapper is not None:
              if _username is not None:
                  _username = self._field_info.username.wrapper(_username)
          self._username = _username
          pass

      def unloadGroupname(self, ):
          self._groupnameLoaded = False
          self._groupname = None;

      def getGroupname(self, current = None):
          self.errorIfUnloaded()
          return self._groupname

      def setGroupname(self, _groupname, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.groupname.wrapper is not None:
              if _groupname is not None:
                  _groupname = self._field_info.groupname.wrapper(_groupname)
          self._groupname = _groupname
          pass

      def unloadType(self, ):
          self._typeLoaded = False
          self._type = None;

      def getType(self, current = None):
          self.errorIfUnloaded()
          return self._type

      def setType(self, _type, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.type.wrapper is not None:
              if _type is not None:
                  _type = self._field_info.type.wrapper(_type)
          self._type = _type
          pass

      def unloadMessage(self, ):
          self._messageLoaded = False
          self._message = None;

      def getMessage(self, current = None):
          self.errorIfUnloaded()
          return self._message

      def setMessage(self, _message, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.message.wrapper is not None:
              if _message is not None:
                  _message = self._field_info.message.wrapper(_message)
          self._message = _message
          pass

      def unloadStatus(self, ):
          self._statusLoaded = False
          self._status = None;

      def getStatus(self, current = None):
          self.errorIfUnloaded()
          return self._status

      def setStatus(self, _status, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.status.wrapper is not None:
              if _status is not None:
                  _status = self._field_info.status.wrapper(_status)
          self._status = _status
          pass

      def unloadSubmitted(self, ):
          self._submittedLoaded = False
          self._submitted = None;

      def getSubmitted(self, current = None):
          self.errorIfUnloaded()
          return self._submitted

      def setSubmitted(self, _submitted, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.submitted.wrapper is not None:
              if _submitted is not None:
                  _submitted = self._field_info.submitted.wrapper(_submitted)
          self._submitted = _submitted
          pass

      def unloadScheduledFor(self, ):
          self._scheduledForLoaded = False
          self._scheduledFor = None;

      def getScheduledFor(self, current = None):
          self.errorIfUnloaded()
          return self._scheduledFor

      def setScheduledFor(self, _scheduledFor, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.scheduledFor.wrapper is not None:
              if _scheduledFor is not None:
                  _scheduledFor = self._field_info.scheduledFor.wrapper(_scheduledFor)
          self._scheduledFor = _scheduledFor
          pass

      def unloadStarted(self, ):
          self._startedLoaded = False
          self._started = None;

      def getStarted(self, current = None):
          self.errorIfUnloaded()
          return self._started

      def setStarted(self, _started, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.started.wrapper is not None:
              if _started is not None:
                  _started = self._field_info.started.wrapper(_started)
          self._started = _started
          pass

      def unloadFinished(self, ):
          self._finishedLoaded = False
          self._finished = None;

      def getFinished(self, current = None):
          self.errorIfUnloaded()
          return self._finished

      def setFinished(self, _finished, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.finished.wrapper is not None:
              if _finished is not None:
                  _finished = self._field_info.finished.wrapper(_finished)
          self._finished = _finished
          pass

      def unloadOriginalFileLinks(self, current = None):
          self._originalFileLinksLoaded = False
          self._originalFileLinksSeq = None;

      def _getOriginalFileLinks(self, current = None):
          self.errorIfUnloaded()
          return self._originalFileLinksSeq

      def _setOriginalFileLinks(self, _originalFileLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.originalFileLinksSeq.wrapper is not None:
              if _originalFileLinks is not None:
                  _originalFileLinks = self._field_info.originalFileLinksSeq.wrapper(_originalFileLinks)
          self._originalFileLinksSeq = _originalFileLinks
          self.checkUnloadedProperty(_originalFileLinks,'originalFileLinksLoaded')

      def isOriginalFileLinksLoaded(self):
          return self._originalFileLinksLoaded

      def sizeOfOriginalFileLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: return -1
          return len(self._originalFileLinksSeq)

      def copyOriginalFileLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          return list(self._originalFileLinksSeq)

      def iterateOriginalFileLinks(self):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          return iter(self._originalFileLinksSeq)

      def addJobOriginalFileLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          self._originalFileLinksSeq.append( target );
          target.setParent( self )

      def addAllJobOriginalFileLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          self._originalFileLinksSeq.extend( targets )
          for target in targets:
              target.setParent( self )

      def removeJobOriginalFileLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          self._originalFileLinksSeq.remove( target )
          target.setParent( None )

      def removeAllJobOriginalFileLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          for elt in targets:
              elt.setParent( None )
              self._originalFileLinksSeq.remove( elt )

      def clearOriginalFileLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          for elt in self._originalFileLinksSeq:
              elt.setParent( None )
          self._originalFileLinksSeq = list()

      def reloadOriginalFileLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._originalFileLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: originalFileLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyOriginalFileLinks() # May also throw
          for elt in copy:
              elt.setParent( self )
          self._originalFileLinksSeq = copy
          toCopy.unloadOriginalFileLinks()
          self._originalFileLinksLoaded = True

      def getOriginalFileLinksCountPerOwner(self, current = None):
          return self._originalFileLinksCountPerOwner

      def linkOriginalFile(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          link = _omero_model.JobOriginalFileLinkI()
          link.link( self, addition );
          self.addJobOriginalFileLinkToBoth( link, True )
          return link

      def addJobOriginalFileLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          self._originalFileLinksSeq.append( link )

      def findJobOriginalFileLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          result = list()
          for link in self._originalFileLinksSeq:
              if link.getChild() == removal: result.append(link)
          return result

      def unlinkOriginalFile(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          toRemove = self.findJobOriginalFileLink(removal)
          for next in toRemove:
              self.removeJobOriginalFileLinkFromBoth( next, True )

      def removeJobOriginalFileLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._originalFileLinksLoaded: self.throwNullCollectionException("originalFileLinksSeq")
          self._originalFileLinksSeq.remove( link )

      def linkedOriginalFileList(self, current = None):
          self.errorIfUnloaded()
          if not self.originalFileLinksLoaded: self.throwNullCollectionException("OriginalFileLinks")
          linked = []
          for link in self._originalFileLinksSeq:
              linked.append( link.getChild() )
          return linked


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.ThumbnailGenerationJobI = ThumbnailGenerationJobI



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2015 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Utilities for manipulating bulk-annotations

Includes classes to help with basic data-munging (TODO), and for formatting
data for clients.
"""

from builtins import range
from past.builtins import basestring
from builtins import object
from collections import deque, OrderedDict
from omero.constants import namespaces
import re

# TODO: Make jinja2 mandatory?
try:
    import jinja2
    JINJA2_MISSING = None
except ImportError as j2exc:
    JINJA2_MISSING = j2exc


try:
    # Namespace for Bulk-Annotations configuration files
    NSBULKANNOTATIONSCONFIG = namespaces.NSBULKANNOTATIONS + "/config"
    # Namespace for raw input for a Bulk-Annotations table
    NSBULKANNOTATIONSRAW = namespaces.NSBULKANNOTATIONS + "/raw"
except Exception:
    pass





class GroupConfig(object):

    def __init__(self, namespace, column_cfg):
        self.namespace = namespace
        self.columns = column_cfg

    def __eq__(self, other):
        return (isinstance(other, self.__class__)
                and self.__dict__ == other.__dict__)


class BulkAnnotationConfiguration(object):
    """
    Parent class for handling bulk-annotation column configurations
    """

    REQUIRED = set(["name"])
    OPTIONAL = set([
        "clientname",
        "clientvalue",
        "includeclient",
        "position",
        "include",
        "split",
        "type",
        "visible",
        "omitempty",
        ])
    GROUPREQUIRED = set(["namespace", "columns"])

    def __init__(self, default_cfg, column_cfgs):
        """
        :param default_cfg: Dict of default values, can be empty
        :params column_cfgs: Array of dicts of column configurations
        """
        self.default_cfg = self.get_default_cfg(default_cfg)
        self.column_cfgs = []
        self.group_cfgs = []
        if column_cfgs:
            for c in column_cfgs:
                cfg = self.get_column_config(c)
                if isinstance(cfg, GroupConfig):
                    self.group_cfgs.append(cfg)
                else:
                    self.column_cfgs.append(cfg)

    @staticmethod
    def get_default_cfg(cfg):
        """
        Get the default column configuration, fill in unspecified fields
        """
        default_defaults = {
            "clientvalue": None,
            "includeclient": True,
            "include": True,
            "split": False,
            "type": "string",
            "visible": True,
            "position": -1,
            "clientname": None,
            "omitempty": False
        }
        if not cfg:
            cfg = {}

        invalid = set(cfg.keys()).difference(list(default_defaults.keys()))
        if invalid:
            raise Exception(
                "Invalid key(s) in column defaults: %s" % list(invalid))

        defaults = default_defaults.copy()
        defaults.update(cfg)
        return defaults

    @classmethod
    def validate_column_config(cls, cfg):
        """
        Check whether a column config section is valid, throws Exception if not
        """

        keys = set(cfg.keys())

        missing = cls.REQUIRED.difference(keys)
        if missing:
            raise Exception(
                "Required key(s) missing from column configuration: %s" %
                list(missing))

        if not cfg["name"]:
            raise Exception("Empty name in column configuration: %s" % cfg)

        invalid = keys.difference(cls.OPTIONAL.union(cls.REQUIRED))
        if invalid:
            raise Exception(
                "Invalid key(s) in column configuration: %s" % list(invalid))

    @classmethod
    def validate_filled_column_config(cls, cfg):
        """
        Check whether a column config section is valid after filling all
        optional fields with defaults
        """
        cls.validate_column_config(cfg)
        allfields = cls.OPTIONAL.union(cls.REQUIRED)
        keys = set(cfg.keys())

        missing = allfields.difference(keys)
        if missing:
            raise Exception("Required key(s) missing from column+defaults "
                            "configuration: %s" % list(missing))

        if cfg["includeclient"] and not cfg["include"]:
            raise Exception("Option `includeclient` requires option `include`")

        if not isinstance(cfg["position"], int):
            raise Exception("Option `position` must be an int")

        if cfg["clientvalue"]:
            subbed = re.sub(r"\{\{\s*value\s*\}\}", '', cfg["clientvalue"])
            m = re.search(r"\{\{[\s\w]*}\}", subbed)
            if m:
                raise Exception(
                    "clientvalue template parameter not found: %s" % m.group())

    @classmethod
    def validate_group_config(cls, cfg):
        """
        Check whether a group config section is valid, throws Exception if not
        Not recursive (doesn't check columns for validity)
        """

        keys = set(cfg.keys())

        missing = cls.GROUPREQUIRED.difference(keys)
        if missing:
            raise Exception(
                "Required key(s) missing from group configuration: %s" %
                list(missing))

        if not cfg["namespace"]:
            raise Exception("Empty name in group configuration: %s" % cfg)

        if not cfg["columns"]:
            raise Exception("Empty columns in group configuration: %s" % cfg)

        invalid = keys.difference(cls.GROUPREQUIRED)
        if invalid:
            raise Exception(
                "Invalid key(s) in group configuration: %s" % list(invalid))

    def get_column_config(self, cfg):
        """
        Replace unspecified fields in a column config with defaults
        If this is a group return a GroupConfig object
        """
        if 'group' in cfg:
            gcfg = cfg['group']
            self.validate_group_config(gcfg)
            column_cfgs = [
                self.get_column_config(gc) for gc in gcfg['columns']]
            return GroupConfig(gcfg['namespace'], column_cfgs)

        self.validate_column_config(cfg)
        column_cfg = self.default_cfg.copy()
        column_cfg.update(cfg)
        self.validate_filled_column_config(column_cfg)
        return column_cfg


class KeyValueListPassThrough(object):
    """
    Converts bulk-annotation rows into key-value lists without any
    transformation
    """

    def __init__(self, headers, default_cfg=None, column_cfgs=None):
        """
        :param headers: A list of table headers
        """
        self.headers = headers

    def transform(self, rowvalues):
        """
        Pass through a table row unchanged
        :param values: A table rows
        :return: A row in the form [(k1, v1), (k2, v2), ...]
        """
        assert len(rowvalues) == len(self.headers)
        return rowvalues


class KeyValueGroupList(BulkAnnotationConfiguration):
    """
    Converts bulk-annotation rows into key-value lists
    """

    def __init__(self, headers, default_cfg, column_cfgs):
        """
        :param headers: A list of table headers
        """
        # TODO: decide what to do with unmentioned columns
        super(KeyValueGroupList, self).__init__(
            default_cfg, column_cfgs)
        self.headers = headers
        self.headerindexmap = OrderedDict(
            (b, a) for (a, b) in enumerate(self.headers))
        self.checked = set()
        self.output_configs = self.get_output_configs()

    def get_output_configs(self):
        """
        Get the full set of output column configs including column groups
        The default set of column configs has an empty namespace
        """

        # First process groups in case the default is to include all
        # columns not explicitly specified
        output_configs = []
        for gcfg in self.group_cfgs:
            output_cfg = self.get_group_output_configs(gcfg.columns, False)
            output_configs.append(GroupConfig(gcfg.namespace, output_cfg))

        output_defcfg = self.get_group_output_configs(self.column_cfgs, True)
        output_configs.append(GroupConfig('', output_defcfg))
        return output_configs

    def get_group_output_configs(self, column_cfgs, isdefault):
        """
        Get the full set of output column configs for a single group,
        taking into account specified column positions and columns
        included/excluded according to the defaults:

        - positioned columns are at the specified index (1-based)
        - gaps between positioned columns are filled with unpositioned
          columns in order of
          - Configured but unpositioned columns
          - Unconfigured columns in order of headers (assuming the default \
            config is for them to be included)
        - If there are gaps and no remaining columns to be included raise
          an exception
        """
        positioned = {}
        unpositioned = deque()

        # Specified columns
        for cfg in column_cfgs:
            self.checked.add(cfg["name"])
            if not cfg["include"]:
                continue

            pos = cfg["position"]
            if pos > 0:
                if pos in positioned:
                    raise Exception(
                        "Multiple columns specified for position: %d" % pos)
                positioned[pos] = (cfg, self.headerindexmap[cfg["name"]])
            else:
                unpositioned.append((cfg, self.headerindexmap[cfg["name"]]))

        # Unspecified Columns
        if isdefault and self.default_cfg["include"]:
            for name in self.headerindexmap.keys():
                print(name)
                if name not in self.checked:
                    cfg = self.get_column_config({"name": name})
                    assert not isinstance(cfg, GroupConfig)
                    unpositioned.append(
                        (cfg, self.headerindexmap[cfg["name"]]))

        # The dance- put positioned columns in the right place and fill
        # any gaps with unpositioned columns (otherwise append to end)
        output_configs = []
        if positioned:
            for pos in range(1, max(positioned.keys()) + 1):
                if pos in positioned:
                    output_configs.append(positioned.pop(pos))
                elif not unpositioned:
                    raise Exception("No column found for position: %d" % pos)
                else:
                    output_configs.append(unpositioned.popleft())
        output_configs.extend(unpositioned)
        return output_configs

    def get_transformers(self):
        """
        Return a set of KeyValueListTransformer objects, one for each group
        """
        transformers = [KeyValueListTransformer(
            self.headers, gc.columns, gc.namespace)
            for gc in self.output_configs]
        return transformers


class KeyValueListTransformer(object):
    """
    Converts bulk-annotation rows into key-value lists
    """

    def __init__(self, headers, output_configs, name=None):
        """
        :param headers: A list of table headers
        :param output_configs: A list of output configurations
        :param name: The name for this group of keys/values, optional
        """
        self.headers = headers
        self.output_configs = output_configs
        self.name = name

    @staticmethod
    def transform1(value, cfg):
        """
        Process a single value corresponding to a single table row-column

        :return: The key and a list of [value]. In general [values] will be
                 a single item unless `split` is specified in cfg in which
                 case there may be multiple.
        """

        def valuesub(v, cv):
            if JINJA2_MISSING:
                raise JINJA2_MISSING
            t = jinja2.Template(cv)
            return t.render(value=v)

        key = cfg["name"]
        if cfg["clientname"]:
            key = cfg["clientname"]

        if not cfg["visible"]:
            key = "__%s" % key

        if cfg["split"]:
            values = [v.strip() for v in value.split(cfg["split"])]
        else:
            values = [value]

        if cfg["omitempty"]:
            values = [v for v in values if v is not None and (
                not isinstance(v, basestring) or v.strip())]

        if cfg["clientvalue"] is not None:
            values = [valuesub(v, cfg["clientvalue"]) for v in values]
        return key, values

    def transform(self, rowvalues):
        """
        Transform a table row
        :param rowvalues: A table row
        :return: The transformed rows in the form [(k1, v1), (k2, v2), ...]. \
                 v* will be a list of length: \
                 - 1 in most cases \
                 - 0 if `omitempty=True` and value was empty \
                 - 1+ if `split` option is enabled
        """
        assert len(rowvalues) == len(self.headers)
        rowkvs = [self.transform1(rowvalues[i], c)
                  for (c, i) in self.output_configs]
        return rowkvs



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Admin.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_Collections_ice
import omero_System_ice
import omero_cmd_API_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.cmd
_M_omero.cmd = Ice.openModule('omero.cmd')

# Start of module omero
__name__ = 'omero'

# Start of module omero.cmd
__name__ = 'omero.cmd'

if 'ResetPasswordRequest' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.ResetPasswordRequest = Ice.createTempClass()
    class ResetPasswordRequest(_M_omero.cmd.Request):
        """
        Requests a reset password for the given user.
        The user must not be an administrator.
        examples:
        - omero.cmd.ResetPasswordRequest(omename, email)
        sends new password to the given user
        """
        def __init__(self, omename='', email=''):
            _M_omero.cmd.Request.__init__(self)
            self.omename = omename
            self.email = email

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Request', '::omero::cmd::ResetPasswordRequest')

        def ice_id(self, current=None):
            return '::omero::cmd::ResetPasswordRequest'

        def ice_staticId():
            return '::omero::cmd::ResetPasswordRequest'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_ResetPasswordRequest)

        __repr__ = __str__

    _M_omero.cmd.ResetPasswordRequestPrx = Ice.createTempClass()
    class ResetPasswordRequestPrx(_M_omero.cmd.RequestPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.ResetPasswordRequestPrx.ice_checkedCast(proxy, '::omero::cmd::ResetPasswordRequest', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.ResetPasswordRequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::ResetPasswordRequest'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_ResetPasswordRequestPrx = IcePy.defineProxy('::omero::cmd::ResetPasswordRequest', ResetPasswordRequestPrx)

    _M_omero.cmd._t_ResetPasswordRequest = IcePy.defineClass('::omero::cmd::ResetPasswordRequest', ResetPasswordRequest, -1, (), False, False, _M_omero.cmd._t_Request, (), (
        ('omename', (), IcePy._t_string, False, 0),
        ('email', (), IcePy._t_string, False, 0)
    ))
    ResetPasswordRequest._ice_type = _M_omero.cmd._t_ResetPasswordRequest

    _M_omero.cmd.ResetPasswordRequest = ResetPasswordRequest
    del ResetPasswordRequest

    _M_omero.cmd.ResetPasswordRequestPrx = ResetPasswordRequestPrx
    del ResetPasswordRequestPrx

if 'ResetPasswordResponse' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.ResetPasswordResponse = Ice.createTempClass()
    class ResetPasswordResponse(_M_omero.cmd.OK):
        """
        Successful response for ResetPasswordRequest.
        If no valid user with matching email is found,
        an ERR will be returned.
        """
        def __init__(self):
            _M_omero.cmd.OK.__init__(self)

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::OK', '::omero::cmd::ResetPasswordResponse', '::omero::cmd::Response')

        def ice_id(self, current=None):
            return '::omero::cmd::ResetPasswordResponse'

        def ice_staticId():
            return '::omero::cmd::ResetPasswordResponse'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_ResetPasswordResponse)

        __repr__ = __str__

    _M_omero.cmd.ResetPasswordResponsePrx = Ice.createTempClass()
    class ResetPasswordResponsePrx(_M_omero.cmd.OKPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.ResetPasswordResponsePrx.ice_checkedCast(proxy, '::omero::cmd::ResetPasswordResponse', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.ResetPasswordResponsePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::ResetPasswordResponse'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_ResetPasswordResponsePrx = IcePy.defineProxy('::omero::cmd::ResetPasswordResponse', ResetPasswordResponsePrx)

    _M_omero.cmd._t_ResetPasswordResponse = IcePy.defineClass('::omero::cmd::ResetPasswordResponse', ResetPasswordResponse, -1, (), False, False, _M_omero.cmd._t_OK, (), ())
    ResetPasswordResponse._ice_type = _M_omero.cmd._t_ResetPasswordResponse

    _M_omero.cmd.ResetPasswordResponse = ResetPasswordResponse
    del ResetPasswordResponse

    _M_omero.cmd.ResetPasswordResponsePrx = ResetPasswordResponsePrx
    del ResetPasswordResponsePrx

if 'UpdateSessionTimeoutRequest' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.UpdateSessionTimeoutRequest = Ice.createTempClass()
    class UpdateSessionTimeoutRequest(_M_omero.cmd.Request):
        """
        Proposes a change to one or both of the timeToLive
        and timeToIdle properties of a live session. The session
        uuid cannot be null. If either other argument is null,
        it will be ignored. Otherwise, the long value will be
        interpreted as the the millisecond value which should
        be set. Non-administrators will not be able to reduce
        current values. No special response is returned, but
        an omero.cmd.OK counts as success.
        """
        def __init__(self, session='', timeToLive=None, timeToIdle=None):
            _M_omero.cmd.Request.__init__(self)
            self.session = session
            self.timeToLive = timeToLive
            self.timeToIdle = timeToIdle

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::Request', '::omero::cmd::UpdateSessionTimeoutRequest')

        def ice_id(self, current=None):
            return '::omero::cmd::UpdateSessionTimeoutRequest'

        def ice_staticId():
            return '::omero::cmd::UpdateSessionTimeoutRequest'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_UpdateSessionTimeoutRequest)

        __repr__ = __str__

    _M_omero.cmd.UpdateSessionTimeoutRequestPrx = Ice.createTempClass()
    class UpdateSessionTimeoutRequestPrx(_M_omero.cmd.RequestPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.UpdateSessionTimeoutRequestPrx.ice_checkedCast(proxy, '::omero::cmd::UpdateSessionTimeoutRequest', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.UpdateSessionTimeoutRequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::UpdateSessionTimeoutRequest'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_UpdateSessionTimeoutRequestPrx = IcePy.defineProxy('::omero::cmd::UpdateSessionTimeoutRequest', UpdateSessionTimeoutRequestPrx)

    _M_omero.cmd._t_UpdateSessionTimeoutRequest = IcePy.declareClass('::omero::cmd::UpdateSessionTimeoutRequest')

    _M_omero.cmd._t_UpdateSessionTimeoutRequest = IcePy.defineClass('::omero::cmd::UpdateSessionTimeoutRequest', UpdateSessionTimeoutRequest, -1, (), False, False, _M_omero.cmd._t_Request, (), (
        ('session', (), IcePy._t_string, False, 0),
        ('timeToLive', (), _M_omero._t_RLong, False, 0),
        ('timeToIdle', (), _M_omero._t_RLong, False, 0)
    ))
    UpdateSessionTimeoutRequest._ice_type = _M_omero.cmd._t_UpdateSessionTimeoutRequest

    _M_omero.cmd.UpdateSessionTimeoutRequest = UpdateSessionTimeoutRequest
    del UpdateSessionTimeoutRequest

    _M_omero.cmd.UpdateSessionTimeoutRequestPrx = UpdateSessionTimeoutRequestPrx
    del UpdateSessionTimeoutRequestPrx

if 'CurrentSessionsRequest' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.CurrentSessionsRequest = Ice.createTempClass()
    class CurrentSessionsRequest(_M_omero.cmd.Request):
        """
        Argument-less request that will produce a
        CurrentSessionsResponse if no omero.cmd.ERR occurs.
        """
        def __init__(self):
            _M_omero.cmd.Request.__init__(self)

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::CurrentSessionsRequest', '::omero::cmd::Request')

        def ice_id(self, current=None):
            return '::omero::cmd::CurrentSessionsRequest'

        def ice_staticId():
            return '::omero::cmd::CurrentSessionsRequest'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_CurrentSessionsRequest)

        __repr__ = __str__

    _M_omero.cmd.CurrentSessionsRequestPrx = Ice.createTempClass()
    class CurrentSessionsRequestPrx(_M_omero.cmd.RequestPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.CurrentSessionsRequestPrx.ice_checkedCast(proxy, '::omero::cmd::CurrentSessionsRequest', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.CurrentSessionsRequestPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::CurrentSessionsRequest'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_CurrentSessionsRequestPrx = IcePy.defineProxy('::omero::cmd::CurrentSessionsRequest', CurrentSessionsRequestPrx)

    _M_omero.cmd._t_CurrentSessionsRequest = IcePy.defineClass('::omero::cmd::CurrentSessionsRequest', CurrentSessionsRequest, -1, (), False, False, _M_omero.cmd._t_Request, (), ())
    CurrentSessionsRequest._ice_type = _M_omero.cmd._t_CurrentSessionsRequest

    _M_omero.cmd.CurrentSessionsRequest = CurrentSessionsRequest
    del CurrentSessionsRequest

    _M_omero.cmd.CurrentSessionsRequestPrx = CurrentSessionsRequestPrx
    del CurrentSessionsRequestPrx

if 'CurrentSessionsResponse' not in _M_omero.cmd.__dict__:
    _M_omero.cmd.CurrentSessionsResponse = Ice.createTempClass()
    class CurrentSessionsResponse(_M_omero.cmd.OK):
        """
        Return value from omero.cmd.CurrentSessionsRequest
        consisting of two ordered lists of matching length. The sessions
        field contains a list of the OMERO omero.model.Session
        objects that are currently active *after* all timeouts have been
        applied.
        This is the value that would be returned by
        {@code omero.api.ISession.getSession} when joined to that session.
        Similarly, the contexts field contains the value that would be
        returned by a call to {@code omero.api.IAdmin.getEventContext}.
        For non-administrators, most values for all sessions other than
        those belonging to that user will be null.
        Members:
        sessions -- omero.model.Session objects loaded from
        the database.
        contexts -- omero.sys.EventContext objects stored in
        memory by the server.
        data -- Other session state which may vary based on
        usage. This may include "hitCount", "lastAccess",
        and similar metrics.
        """
        def __init__(self, sessions=None, contexts=None, data=None):
            _M_omero.cmd.OK.__init__(self)
            self.sessions = sessions
            self.contexts = contexts
            self.data = data

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::cmd::CurrentSessionsResponse', '::omero::cmd::OK', '::omero::cmd::Response')

        def ice_id(self, current=None):
            return '::omero::cmd::CurrentSessionsResponse'

        def ice_staticId():
            return '::omero::cmd::CurrentSessionsResponse'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.cmd._t_CurrentSessionsResponse)

        __repr__ = __str__

    _M_omero.cmd.CurrentSessionsResponsePrx = Ice.createTempClass()
    class CurrentSessionsResponsePrx(_M_omero.cmd.OKPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.cmd.CurrentSessionsResponsePrx.ice_checkedCast(proxy, '::omero::cmd::CurrentSessionsResponse', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.cmd.CurrentSessionsResponsePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::cmd::CurrentSessionsResponse'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.cmd._t_CurrentSessionsResponsePrx = IcePy.defineProxy('::omero::cmd::CurrentSessionsResponse', CurrentSessionsResponsePrx)

    _M_omero.cmd._t_CurrentSessionsResponse = IcePy.declareClass('::omero::cmd::CurrentSessionsResponse')

    _M_omero.cmd._t_CurrentSessionsResponse = IcePy.defineClass('::omero::cmd::CurrentSessionsResponse', CurrentSessionsResponse, -1, (), False, False, _M_omero.cmd._t_OK, (), (
        ('sessions', (), _M_omero.api._t_SessionList, False, 0),
        ('contexts', (), _M_omero.api._t_EventContextList, False, 0),
        ('data', (), _M_omero.api._t_RTypeDictArray, False, 0)
    ))
    CurrentSessionsResponse._ice_type = _M_omero.cmd._t_CurrentSessionsResponse

    _M_omero.cmd.CurrentSessionsResponse = CurrentSessionsResponse
    del CurrentSessionsResponse

    _M_omero.cmd.CurrentSessionsResponsePrx = CurrentSessionsResponsePrx
    del CurrentSessionsResponsePrx

# End of module omero.cmd

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Filament.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice
import omero_model_LightSource_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'FilamentType' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilamentType = IcePy.declareClass('::omero::model::FilamentType')
    _M_omero.model._t_FilamentTypePrx = IcePy.declareProxy('::omero::model::FilamentType')

if 'Power' not in _M_omero.model.__dict__:
    _M_omero.model._t_Power = IcePy.declareClass('::omero::model::Power')
    _M_omero.model._t_PowerPrx = IcePy.declareProxy('::omero::model::Power')

if 'Instrument' not in _M_omero.model.__dict__:
    _M_omero.model._t_Instrument = IcePy.declareClass('::omero::model::Instrument')
    _M_omero.model._t_InstrumentPrx = IcePy.declareProxy('::omero::model::Instrument')

if 'LightSourceAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_LightSourceAnnotationLink = IcePy.declareClass('::omero::model::LightSourceAnnotationLink')
    _M_omero.model._t_LightSourceAnnotationLinkPrx = IcePy.declareProxy('::omero::model::LightSourceAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'Filament' not in _M_omero.model.__dict__:
    _M_omero.model.Filament = Ice.createTempClass()
    class Filament(_M_omero.model.LightSource):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _manufacturer=None, _model=None, _power=None, _lotNumber=None, _serialNumber=None, _instrument=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _type=None):
            if Ice.getType(self) == _M_omero.model.Filament:
                raise RuntimeError('omero.model.Filament is an abstract class')
            _M_omero.model.LightSource.__init__(self, _id, _details, _loaded, _version, _manufacturer, _model, _power, _lotNumber, _serialNumber, _instrument, _annotationLinksSeq, _annotationLinksLoaded, _annotationLinksCountPerOwner)
            self._type = _type

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::Filament', '::omero::model::IObject', '::omero::model::LightSource')

        def ice_id(self, current=None):
            return '::omero::model::Filament'

        def ice_staticId():
            return '::omero::model::Filament'
        ice_staticId = staticmethod(ice_staticId)

        def getType(self, current=None):
            pass

        def setType(self, theType, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Filament)

        __repr__ = __str__

    _M_omero.model.FilamentPrx = Ice.createTempClass()
    class FilamentPrx(_M_omero.model.LightSourcePrx):

        def getType(self, _ctx=None):
            return _M_omero.model.Filament._op_getType.invoke(self, ((), _ctx))

        def begin_getType(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Filament._op_getType.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getType(self, _r):
            return _M_omero.model.Filament._op_getType.end(self, _r)

        def setType(self, theType, _ctx=None):
            return _M_omero.model.Filament._op_setType.invoke(self, ((theType, ), _ctx))

        def begin_setType(self, theType, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Filament._op_setType.begin(self, ((theType, ), _response, _ex, _sent, _ctx))

        def end_setType(self, _r):
            return _M_omero.model.Filament._op_setType.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.FilamentPrx.ice_checkedCast(proxy, '::omero::model::Filament', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.FilamentPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Filament'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_FilamentPrx = IcePy.defineProxy('::omero::model::Filament', FilamentPrx)

    _M_omero.model._t_Filament = IcePy.declareClass('::omero::model::Filament')

    _M_omero.model._t_Filament = IcePy.defineClass('::omero::model::Filament', Filament, -1, (), True, False, _M_omero.model._t_LightSource, (), (('_type', (), _M_omero.model._t_FilamentType, False, 0),))
    Filament._ice_type = _M_omero.model._t_Filament

    Filament._op_getType = IcePy.Operation('getType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_FilamentType, False, 0), ())
    Filament._op_setType = IcePy.Operation('setType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilamentType, False, 0),), (), None, ())

    _M_omero.model.Filament = Filament
    del Filament

    _M_omero.model.FilamentPrx = FilamentPrx
    del FilamentPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# Generated by slice2py - DO NOT EDIT!
#

import Ice
Ice.updateModule("omero.constants")

# Modules:
import omero_Constants_ice
import omero_api_MetadataStore_ice

# Submodules:
from . import cluster
from . import annotation
from . import data
from . import metadata
from . import namespaces
from . import analysis
from . import jobs
from . import permissions
from . import projection
from . import topics
from . import categories
